/**
 * 
 */
package edx.actions;

import java.io.ByteArrayOutputStream;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.jboss.logging.Logger;
import org.jboss.soa.esb.actions.AbstractActionPipelineProcessor;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;

import edx.datamodel.Record;
import edx.datamodel.ObjectFactory;

/**
 * This class do logging of the message content via logger which has  
 * the category provided by config attribute "log.category".
 * 
 * @author     Edward Pilipczuk
 */
public class MsgLogAction extends AbstractActionPipelineProcessor {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6960519699120106470L;
	
	private Logger log = null;
	private ConfigTree _config = null;
	
	/**
	 * constructor 
	 */
	public MsgLogAction( ConfigTree config ) {
		_config = config;
		String cat = _config.getAttribute("log.category","EDX");
		log = Logger.getLogger(cat);
	}

	public Message noOperation(Message message) { return message; } 

	@SuppressWarnings("unchecked")
	public Message process(Message message) throws ActionProcessingException {
		logString( new String(message.getHeader().toString()) );
		Object body = message.getBody().get();
		if( body instanceof Map ) 
			logMap((Map<String,Object>)body);
		else
			logObject("Message body [[","]]",body);
		return message;         	
	}
	  		
	private void logString( String logdata ) {
		log.info(logdata);
	}
	
	private void logMap( Map<String,Object> mapData ) {
		Set<String> ks = (Set<String>)mapData.keySet();
		for( String key: ks ) {
			logString( "Key='"+key+"'" );
			logObject( "Value=[[", "]]", mapData.get(key) );
		}
	}
	
	@SuppressWarnings("unchecked")
	private void logObject( String pfx, String sfx, Object obj ) {

		String prt = pfx;
		
		if( obj instanceof String ) {
			prt += (String)obj;
		}
		else if( obj instanceof byte[] )
			prt += new String((byte[])obj);
		else if( obj instanceof Integer )
			prt += (Integer)obj;
		else if( obj instanceof Map ) {
			logMap((Map<String, Object>) obj);
		}
		else if( obj instanceof Record ) {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			try {
				String pathname = obj.getClass().getPackage().getName(); 
				Marshaller m = JAXBContext.newInstance(pathname, this.getClass().getClassLoader()).createMarshaller();
				ObjectFactory rof = new ObjectFactory();	
				m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
				m.marshal(rof.createDatarecord((Record)obj),bos);
				prt += bos.toString();
			} catch (JAXBException e) {
				prt += "!!! Cannot marshall DataRecord object, JAXBException: "+e.getMessage();
			}
		}
		else {
			prt += "--- Don't know how to print the object "+((obj!=null)?obj.getClass().getSimpleName():"(null)")+" ---";
		}
		
		prt += sfx;
		logString( prt );
	}
}
