/**
 * 
 */
package edx.actions;

import java.io.StringReader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.jboss.logging.Logger;
import org.jboss.soa.esb.actions.AbstractActionPipelineProcessor;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;

import edx.datamodel.Record;

/**
 * The class provides parsing XML documents to java objects of Record class 
 * <br/><br/>
 * @author Edward Pilipczuk
 */
public class ParseRecord extends AbstractActionPipelineProcessor {

	private static final long serialVersionUID = 6960519699120106470L;
	private Logger log = Logger.getLogger(this.getClass().getSimpleName());

	/**
	 *	constructor  
	 */
	public ParseRecord(ConfigTree config) {
	}

	/* (non-Javadoc)
	 * @see org.jboss.soa.esb.actions.ActionPipelineProcessor#process(org.jboss.soa.esb.message.Message)
	 */
	@SuppressWarnings("unchecked")
	public Message process(Message message) throws ActionProcessingException {
		
		// Get JMS Correlation Id fragment from RelatesTo field of Message header
		String msgId = message.getHeader().getCall().getRelatesTo().getFragment(); 
		log.info("Received Record with Message Correlation ID= "+msgId);
		
		Record record = null;
		try {
			StringReader sr = new StringReader((String)message.getBody().get());
			Unmarshaller um = JAXBContext.newInstance(Record.class.getPackage().getName(), 
					this.getClass().getClassLoader()).createUnmarshaller();
			record = ((JAXBElement<Record>) um.unmarshal(sr)).getValue();
		} catch(JAXBException e) {
			String ermsg = "Java object of Record type cannot be bound to XML document, JAXBException: "+e.getMessage(); 
			log.warn(ermsg);
			throw new ActionProcessingException(ermsg);
		}

		// replace the message payload by the Record object
		message.getBody().add(record);
		// inform that record with the given GUID has been correctly generated 
		log.info("Record ID="+msgId+" parsed, seq="+record.getSeq());
	
		return message;
	}	
	
}
